import logging
# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.

class ColoredFormatter(logging.Formatter):
    COLORS = {
        'DEBUG': '\033[94m',  # Blue
        'INFO': '\033[92m',  # Green
        'WARNING': '\033[93m',  # Yellow
        'ERROR': '\033[91m',  # Red
        'CRITICAL': '\033[95m',  # Magenta
        'RESET': '\033[0m',  # Reset
    }

    def format(self, record):
        levelname = record.levelname
        message = super().format(record)
        color = self.COLORS.get(levelname, self.COLORS['RESET'])
        return color + message + self.COLORS['RESET']
    
def set_log_color_level(level):
    logger = logging.getLogger()
    logger.setLevel(level)

    # Check if there's already a console handler
    if not any(isinstance(handler, logging.StreamHandler) for handler in logger.handlers):
        # Create console handler
        console_handler = logging.StreamHandler()
        console_handler.setLevel(level)

        # Create formatter and add it to the handlers
        formatter = ColoredFormatter('%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - %(message)s\n')
        console_handler.setFormatter(formatter)

        # Add the handlers to the logger
        logger.addHandler(console_handler)

    # Prevent log messages from propagating to the root logger
    logger.propagate = False    

    # Test messages
    logger.debug("This is a debug message")
    logger.info("This is an info message")
    logger.warning("This is a warning message")
    logger.error("This is an error message")
    logger.critical("This is a critical message")


# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import unittest
import json
from didall.utils.did_generate import did_generate, sign_did_document_secp256r1
from didall.utils.did_verify import verify_did_document, extract_public_key
from cryptography.hazmat.primitives.asymmetric import ec
import logging

class TestDIDManagement(unittest.TestCase):

    def test_did_generation_and_verification(self):
        # Generate DID document and keys
        private_key, public_key, did, did_document_json = did_generate("wss://example.com/endpoint")
        did_document = json.loads(did_document_json)

        # Check if the DID document contains the correct DID
        self.assertIn(did, did_document_json)

        # Verify DID document
        result, message = verify_did_document(did_document)
        self.assertTrue(result, msg="DID document verification failed: " + message)

        # Test signature functionality
        signed_did_document = sign_did_document_secp256r1(private_key, did_document)
        self.assertIn('proofValue', signed_did_document['proof'], "Signature not properly added to DID document")

        # Verify signature again
        result, message = verify_did_document(signed_did_document)
        self.assertTrue(result, msg="Re-signed DID document verification failed: " + message)
        logging.info("DID document verification passed")

    def test_public_key_extraction(self):
        # Generate DID document and keys
        private_key, public_key, did, did_document_json = did_generate("wss://example.com/endpoint")
        did_document = json.loads(did_document_json)

        # Extract public key
        extracted_public_key = extract_public_key(did_document, f"{did}#keys-1")
        self.assertIsInstance(extracted_public_key, ec.EllipticCurvePublicKey, "Extracted public key type is incorrect")

        logging.info("Public key extraction successful")


if __name__ == '__main__':
    unittest.main()




# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import asyncio
import json
import logging
from typing import Callable, Tuple, Union
from didall.message_generation import generate_encrypted_message
from didall.wss_message_client import WssMessageClient
from didall.short_term_key_generater import ShortTermKeyGenerater
from didall.utils.crypto_tool import decrypt_aes_gcm_sha256, generate_random_hex

# TODO: 
# Key expiration management, check expiration when using
# Key error management
# Handling cases when key is not found or expired during sending
# Handling error responses
class WssMessageSDK:
    def __init__(self, wss_url: str, api_key: str, routers: list[tuple[str, str]], 
                 short_term_key_callback: Callable[[tuple[str, str, str]], None]):
        '''
        Initialize WssMessageSDK class.

        Args:
            wss_url (str): URL of the WebSocket service.
            api_key (str): API key for the WebSocket service.
            routers (list[tuple[str, str]]): List of routers, each tuple contains the private key and did document of the router did
            short_term_key_callback (Callable[[tuple[str, str, str]], None]): Callback function for short-term keys,
                used to callback short-term key info. The tuple contains three strings: local did, remote did, and key info JSON string,
                JSON definition is the same as the input of set_short_term_keys
        '''
        self.client = WssMessageClient(wss_url, api_key)
        self.short_term_keys: dict[str, dict] = {}
        self.short_term_keys_combined: dict[str, dict] = {}
        self.local_did_to_private_key: dict[str, str] = {}  # Add a dictionary to store local DID and corresponding private key
        self.short_term_key_callback = short_term_key_callback
        self.short_term_key_generater_session_dict: dict[str, ShortTermKeyGenerater] = {}

    @classmethod
    async def create(cls, wss_url: str, api_key: str, routers: list[tuple[str, str]], 
                     short_term_key_callback: Callable[[tuple[str, str, str]], None]):
        """Factory method to asynchronously create WssMessageSDK instance and register routers"""
        instance = cls(wss_url, api_key, routers, short_term_key_callback)
        await instance.client.register_routers(routers)
        return instance

    def key_combined(self, local_did: str, remote_did: str) -> str:
        return f"{local_did}_{remote_did}"

    def insert_did_private_key(self, local_did: str, private_key_pem: str):
        """Insert local DID and corresponding private key into the dictionary"""
        self.local_did_to_private_key[local_did] = private_key_pem
        
    # Change input to JSON, communicate with external through JSON, caller doesn't need to know JSON details
    def set_short_term_keys(self, local_did: str, remote_did: str, secret_info_json: str):
        """
        Set short-term key information based on JSON string. Short-term keys can be used before expiration,
        previously negotiated short-term keys can be set if server restarts.
        JSON string contains send encryption key, receive decryption key, key ID, key expiration time and cipher suite.

        Args:
            local_did (str): Local DID.
            remote_did (str): Remote DID.
            secret_info_json (str): JSON string containing key information. Same as negotiate_short_term_keys return value
        """
        secret_info = json.loads(secret_info_json)
        secret_key_id = secret_info['secret_key_id']
        key_combined = self.key_combined(local_did, remote_did)
        self.short_term_keys_combined[key_combined] = {
            "local_did": local_did,
            "remote_did": remote_did,
            "secret_key_id": secret_key_id,
            "send_encryption_key": secret_info['send_encryption_key'],
            "receive_decryption_key": secret_info['receive_decryption_key'],
            "key_expires": secret_info['key_expires'],
            "cipher_suite": secret_info['cipher_suite']
        }
        self.short_term_keys[secret_key_id] = {
            "local_did": local_did,
            "remote_did": remote_did,
            "send_encryption_key": secret_info['send_encryption_key'],
            "receive_decryption_key": secret_info['receive_decryption_key'],
            "key_expires": secret_info['key_expires'],
            "cipher_suite": secret_info['cipher_suite']
        }

    async def negotiate_short_term_keys(self, local_did: str, 
                                        did_private_key_pem: str, 
                                        remote_did: str) -> str:
        """
        Negotiate short-term keys and return JSON string containing key information.
        If negotiation is successful, return JSON string containing local DID, remote DID and key information;
        If negotiation fails, return JSON string containing error information.

        Args:
            local_did (str): Local DID.
            did_private_key_pem (str): Private key of local DID, in PEM format.
            remote_did (str): Remote DID.

        Returns:
            str: Key information JSON string. Generated JSON field description:
                send_encryption_key: Encryption key used by sender, represented as hexadecimal string.
                receive_decryption_key: Decryption key used by receiver, represented as hexadecimal string.
                secret_key_id: Unique identifier of the key.
                key_expires: Expiration time of the key, represented as Unix timestamp.
                cipher_suite: Name of the encryption suite used.

        Note: After function returns successfully, short_term_key_callback function is no longer called.
        """
        json_send_func = self.client.send_data  # Assume WssMessageClient has a method to send JSON messages
        key_gen = ShortTermKeyGenerater(local_did, did_private_key_pem, remote_did, json_send_func, is_initiator=True)
        
        self.short_term_key_generater_session_dict[key_gen.session_id] = key_gen
        success = await key_gen.generate_short_term_key_active()
        self.short_term_key_generater_session_dict.pop(key_gen.session_id)

        if success:
            remote_did, send_encryption_key, \
            receive_decryption_key, secret_key_id, \
                key_expires, cipher_suite = key_gen.get_final_short_term_key()
            secret_info_json = json.dumps({
                "send_encryption_key": send_encryption_key.hex(),
                "receive_decryption_key": receive_decryption_key.hex(),
                "secret_key_id": secret_key_id,
                "key_expires": key_expires,
                "cipher_suite": cipher_suite
                })
            self.set_short_term_keys(local_did, remote_did, secret_info_json)
            self.short_term_key_callback(local_did, remote_did, secret_info_json)
            return secret_info_json
            
        else:
            logging.error(f"Key negotiation failed: {local_did} -> {remote_did}")
            return None
        
    async def source_hello_process(self, json_data: dict):
        local_did = json_data['destinationDid'] # When receiving hello message from peer, destinationDID is local did
        did_private_key_pem = self.local_did_to_private_key.get(local_did, None)
        if did_private_key_pem is None:
            logging.error(f"Private key for local DID not found: {local_did}")
            return
        
        remote_did = json_data['sourceDid']
        session_id = json_data['sessionId']
        json_send_func = self.client.send_data
        
        key_gen = ShortTermKeyGenerater(local_did, did_private_key_pem, 
                                        remote_did, json_send_func, 
                                        is_initiator=False, session_id=session_id)
        self.short_term_key_generater_session_dict[session_id] = key_gen
        key_gen.receive_json_message(json_data)

        success = await key_gen.generate_short_term_key_passive()
        self.short_term_key_generater_session_dict.pop(key_gen.session_id)
 
        if success:
            remote_did, send_encryption_key, \
            receive_decryption_key, secret_key_id, \
                key_expires, cipher_suite = key_gen.get_final_short_term_key()
            secret_info_json = json.dumps({
                    "send_encryption_key": send_encryption_key.hex(),
                    "receive_decryption_key": receive_decryption_key.hex(),
                    "secret_key_id": secret_key_id,
                    "key_expires": key_expires,
                    "cipher_suite": cipher_suite
                })
            self.set_short_term_keys(local_did, remote_did, secret_info_json)
            self.short_term_key_callback(local_did, remote_did, secret_info_json)
        else:
            logging.error(f"Key negotiation failed: {remote_did} -> {local_did}")


    def ecrypted_message_process(self, json_data: dict):
        
        secret_key_id = json_data['secretKeyId']
        key_info = self.short_term_keys.get(secret_key_id, None)

        if key_info is None:
            logging.error(f"Cannot find secret key info: {secret_key_id}")
            # TODO: Send error message later
            return

        encrypted_data = json_data['encryptedData']
        secret_key = bytes.fromhex(key_info['receive_decryption_key'])
        try:
            plaintext = decrypt_aes_gcm_sha256(encrypted_data, secret_key)
            logging.info(f"Message decryption successful: {plaintext}")
            return plaintext
        except Exception as e:
            logging.error(f"Message decryption failed: {e}")
            return None

    async def recv_data(self) -> Tuple[str, str, str]:
        """Asynchronously receive data"""
        while True:
            json_data = await self.client.receive_data()
            msg_type = json_data['type']
            if msg_type == "sourceHello":
                asyncio.create_task(self.source_hello_process(json_data))
                # TODO: Record the task here. May need to cancel later
            elif msg_type in ["destinationHello", "finished"]:
                session_id = json_data['sessionId']
                if session_id in self.short_term_key_generater_session_dict:
                    self.short_term_key_generater_session_dict[session_id].receive_json_message(json_data)
                else:
                    logging.error(f"Cannot find session_id: {session_id}")
            elif msg_type == 'message':
                msg = self.ecrypted_message_process(json_data)
                if msg is not None:
                    return json_data['sourceDid'], json_data['destinationDid'], msg
            elif msg_type == 'response':
                logging.info(f"Response message: {json_data}")
            else:
                logging.error(f"Unknown message type: {msg_type}")

    async def send_data(self, content: Union[str, bytes], source_did: str, destination_did: str):
        """
        Send encrypted message. Input data can be str or bytes, if it's str, convert to bytes.
        Get key information from short_term_keys_combined, use generate_encrypted_message to create message,
        and call client method to send data.

        Args:
            content (Union[str, bytes]): Message content to send.
            source_did (str): Source DID.
            destination_did (str): Destination DID.

        Returns:
            None
        """
        if isinstance(content, str):
            content = content.encode('utf-8')

        key_combined = self.key_combined(source_did, destination_did)
        key_info = self.short_term_keys_combined.get(key_combined, None)
        if key_info is None:
            # TODO: Handle exception here
            logging.error(f"Key information not found: {key_combined}")
            return
        secret_key_id = key_info['secret_key_id']
        data_secret = bytes.fromhex(key_info['send_encryption_key'])

        encrypted_message = generate_encrypted_message(
            version="1.0",
            message_id=generate_random_hex(16),
            source_did=source_did,
            destination_did=destination_did,
            secret_key_id=secret_key_id,
            data=content,
            data_secret=data_secret
        )

        await self.client.send_data(encrypted_message)


# Example usage
async def main():
    sdk = await WssMessageSDK.create("wss://example.com/ws", "your_api_key", [("private_key_pem1", "did_doc1"), ("private_key_pem2", "did_doc2")], lambda x: print(f"Callback: {x}"))
    source_did, destination_did, data = await sdk.recv_data()
    print(source_did, destination_did, data)
    await sdk.negotiate_short_term_keys("did:example:local", "your_private_key_pem", "did:example:remote")

if __name__ == "__main__":
    asyncio.run(main())



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import unittest
import sys
import os
import asyncio
from unittest.mock import patch, AsyncMock

current_script_path = os.path.abspath(__file__)
current_directory = os.path.dirname(current_script_path)
sys.path.append(current_directory)
sys.path.append(current_directory + "/../")
sys.path.append(current_directory + "/../../")
# Create test loader
loader = unittest.TestLoader()
suite = unittest.TestSuite()

import logging

# Configure logging format
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - %(message)s'
)

# Load all test modules in a directory
suite.addTests(loader.discover(start_dir=current_directory, pattern='test_*.py'))

# Run tests
runner = unittest.TextTestRunner()
runner.run(suite)



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import asyncio
from datetime import datetime
import websockets
import json
import logging

from didall.utils.crypto_tool import generate_random_hex, generate_router_json, load_private_key_from_pem
from cryptography.hazmat.primitives.asymmetric import ec

class WssMessageClient:
    def __init__(self, wss_url: str, api_key: str):
        self.wss_url = wss_url
        self.api_key = api_key
        self.websocket = None
        self.heartbeat_missed = 0
        self.heartbeat_task = None

    async def connect(self):
        """Connect to WebSocket server and authenticate using api_key"""
        try:
            # Prepare authentication header
            headers = {
                "Authorization": f"Bearer {self.api_key}"
            }
            self.websocket = await websockets.connect(self.wss_url, extra_headers=headers)
            self.heartbeat_missed = 0
            if self.heartbeat_task is not None:
                self.heartbeat_task.cancel()
            self.heartbeat_task = asyncio.create_task(self.send_heartbeat())
            logging.info(f"Connected to WebSocket at {self.wss_url} with authorization")
        except Exception as e:
            logging.error(f"Failed to connect to WebSocket: {e}")

    async def receive_data(self) -> dict:
        """Asynchronously receive data from WebSocket server and return parsed JSON dictionary"""
        while True:
            try:
                if self.websocket:
                    logging.info(f"Receiving WSS data: {id(self.websocket)}")
                    data = await self.websocket.recv()
                    json_data = json.loads(data)

                    msg_type = json_data.get("type")
                    if msg_type == "heartbeat" and json_data.get("message") == "pong":
                        self.heartbeat_missed = 0
                        continue

                    logging.info(f"Received[{id(self.websocket)}]: {json_data}")
                    return json_data
                else:
                    logging.warning("WebSocket not connected, attempting to reconnect...")
                    await self.connect()
            except websockets.exceptions.ConnectionClosed:
                logging.error("WebSocket connection closed.")
                self.websocket = None
                await self.connect()
            except json.JSONDecodeError:
                logging.error("Failed to decode JSON from received data.")
            except Exception as e:
                logging.error(f"Error receiving data: {e}")

    async def send_data(self, data: dict):
        """Asynchronously send data to WebSocket server"""
        if self.websocket:
            try:
                await self.websocket.send(json.dumps(data))
                logging.info(f"Data sent to {id(self.websocket)}: {data}")
            except Exception as e:
                logging.error(f"Failed to send data: {e}")
                self.websocket = None
                await self.connect()
        else:
            logging.warning("WebSocket not connected, attempting to reconnect...")
            await self.connect()

    async def send_heartbeat(self):
        """Send a heartbeat message to WebSocket server every 3 seconds"""
        while True:
            if self.websocket:
                message_id = generate_random_hex(16)
                heartbeat_message = {
                    "version": "1.0",
                    "type": "heartbeat",
                    "timestamp": datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
                    "messageId": message_id,
                    "message": "ping"
                }
                await self.websocket.send(json.dumps(heartbeat_message))
                logging.info(f"Heartbeat sent: {heartbeat_message}")
                await asyncio.sleep(3)
                self.heartbeat_missed += 1
                if self.heartbeat_missed >= 3:
                    logging.warning("Heartbeat missed 3 times, attempting to reconnect...")
                    self.websocket = None
                    await self.connect()
            else:
                logging.warning("WebSocket not connected, attempting to reconnect...")
                await self.connect()

    async def register_routers(self, routers: list[tuple[str, str]]):
        if not self.websocket:
            await self.connect()

        routers_json: list[dict] = []
        for router in routers:
            private_key_pem, did_doc = router
            private_key: ec.EllipticCurvePrivateKey = load_private_key_from_pem(private_key_pem)
            router: dict = generate_router_json(private_key, json.loads(did_doc))
            routers_json.append(router)

            message_id = generate_random_hex(16)
            message_json = {
                "version": "1.0",
                "type": "register",
                "timestamp": datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
                "messageId": message_id,
                "routers": routers_json
            }
            await self.websocket.send(json.dumps(message_json))

    async def close(self):
        """Close WebSocket connection"""
        if self.websocket:
            await self.websocket.close()
            logging.info("WebSocket connection closed")



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import unittest
from unittest.mock import patch, AsyncMock, MagicMock
from didall.didallclient import DIDAllClient

class TestDIDAllClient(unittest.TestCase):
    def setUp(self):
        self.did_service_url = "https://example.com/did"
        self.api_key = "test_api_key"
        self.client = DIDAllClient(self.did_service_url, self.api_key)

    @patch('didall.didallclient.did_generate')
    @patch('didall.didallclient.get_pem_from_private_key')
    @patch('aiohttp.ClientSession.post', new_callable=AsyncMock)
    async def test_register_did_document(self, mock_post, mock_get_pem, mock_did_generate):
        # Set up mock function return values
        mock_did_generate.return_value = (None, None, "did:example:123", "{}")
        mock_get_pem.return_value = "mock_pem_key"
        mock_post.return_value.__aenter__.return_value.status = 200
        mock_post.return_value.__aenter__.return_value.text = AsyncMock(return_value="{}")

        # Call the function
        private_key_pem, did, did_document_json = await self.client.generate_register_did_document("wss://example.com/endpoint")

        # Assertions
        self.assertEqual(private_key_pem, "mock_pem_key")
        self.assertEqual(did, "did:example:123")
        self.assertEqual(did_document_json, "{}")
        mock_post.assert_called_once()

    @patch('aiohttp.ClientSession.get', new_callable=AsyncMock)
    async def test_get_did_document(self, mock_get):
        # Set up mock function return values
        mock_get.return_value.__aenter__.return_value.status = 200
        mock_get.return_value.__aenter__.return_value.text = AsyncMock(return_value='{"id": "did:example:123"}')

        # Call the function
        did_document = await self.client.get_did_document("did:example:123")

        # Assertions
        self.assertEqual(did_document, '{"id": "did:example:123"}')
        mock_get.assert_called_once()

    @patch('didall.didallclient.did_generate')
    @patch('didall.didallclient.get_pem_from_private_key')
    @patch('requests.post')
    def test_register_did_document_sync(self, mock_post, mock_get_pem, mock_did_generate):
        # Set up mock function return values
        mock_did_generate.return_value = (None, None, "did:example:123", "{}")
        mock_get_pem.return_value = "mock_pem_key"
        mock_post.return_value.status_code = 200
        mock_post.return_value.text = "{}"

        # Call the function
        private_key_pem, did, did_document_json = self.client.register_did_document_sync("wss://example.com/endpoint")

        # Assertions
        self.assertEqual(private_key_pem, "mock_pem_key")
        self.assertEqual(did, "did:example:123")
        self.assertEqual(did_document_json, "{}")
        mock_post.assert_called_once()

    @patch('requests.get')
    def test_get_did_document_sync(self, mock_get):
        # Set up mock function return values
        mock_get.return_value.status_code = 200
        mock_get.return_value.text = '{"id": "did:example:123"}'

        # Call the function
        did_document = self.client.get_did_document_sync("did:example:123")

        # Assertions
        self.assertEqual(did_document, '{"id": "did:example:123"}')
        mock_get.assert_called_once()

if __name__ == '__main__':
    unittest.main()



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import asyncio
import unittest
import json
from unittest.mock import ANY, patch, AsyncMock

from didall.wss_message_client import WssMessageClient
from didall.utils.crypto_tool import get_pem_from_private_key
from didall.utils.did_generate import did_generate

class TestWssMessageService(unittest.TestCase):
    def setUp(self):
        self.wss_url = "wss://example.com/ws"
        self.api_key = "test_api_key"
        self.wss_service = WssMessageClient(self.wss_url, self.api_key)
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)
    def tearDown(self):
        # Close the event loop at the end of the test
        self.loop.close()
        
    @patch('websockets.connect', new_callable=AsyncMock)
    def test_connect(self, mock_connect):
        """Test if the connect function correctly calls websockets.connect and handles authentication"""
        mock_connect.return_value = AsyncMock()
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.wss_service.connect())
        mock_connect.assert_called_with(self.wss_url, extra_headers={"Authorization": f"Bearer {self.api_key}"})
        self.assertIsNotNone(self.wss_service.websocket)

    @patch('websockets.connect', new_callable=AsyncMock)
    @patch('asyncio.sleep', side_effect=asyncio.CancelledError)
    def test_send_heartbeat(self, mock_sleep, mock_connect):
        """Test if the heartbeat sending function works as expected"""
        mock_websocket = AsyncMock()
        mock_connect.return_value = mock_websocket
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.wss_service.connect())
        with self.assertRaises(asyncio.CancelledError):
            loop.run_until_complete(self.wss_service.send_heartbeat())
        mock_websocket.send.assert_called()

    def test_close(self):
        """Test the connection closing function"""
        self.wss_service.websocket = AsyncMock()
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.wss_service.close())
        self.wss_service.websocket.close.assert_called()

    @patch('utils.did_generate.did_generate')
    def test_register_routers(self, mock_did_generate):
        """Test if the register_routers method correctly handles DID generation and registration"""
        # Set up mock did_generate return value
        mock_private_key = AsyncMock()
        mock_public_key = AsyncMock()
        mock_did = "did:example:123"
        mock_did_document_json = '{"id": "did:example:123"}'
        mock_did_generate.return_value = (mock_private_key, mock_public_key, mock_did, mock_did_document_json)

        # Set WebSocket to AsyncMock
        self.wss_service.websocket = AsyncMock()

        # Call register_routers method
        private_key, public_key, mock_did, did_document_json = did_generate("wss://example.com/endpoint")
        pem_key = get_pem_from_private_key(private_key)
        
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.wss_service.register_routers([(pem_key, did_document_json)]))

        # Construct expected message, not directly using ANY
        expected_message = {
            "version": "1.0",
            "type": "register",
            "routers": [{
                "router": mock_did,
                "proof": {
                    "type": "EcdsaSecp256r1Signature2019",
                    "proofPurpose": "assertionMethod",
                }
            }]
        }

        # Manually verify fields containing ANY
        actual_call_args = self.wss_service.websocket.send.call_args[0][0]
        actual_message = json.loads(actual_call_args)

        # Verify dynamic fields
        self.assertTrue(isinstance(actual_message['timestamp'], str))
        self.assertTrue(isinstance(actual_message['messageId'], str))
        self.assertTrue(isinstance(actual_message['routers'][0]['nonce'], str))
        self.assertTrue(isinstance(actual_message['routers'][0]['proof']['created'], str))
        self.assertTrue(isinstance(actual_message['routers'][0]['proof']['verificationMethod'], str))
        self.assertTrue(isinstance(actual_message['routers'][0]['proof']['proofValue'], str))


if __name__ == '__main__':
    unittest.main()



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import unittest
import sys
import os
import asyncio
from unittest.mock import patch, AsyncMock

# Import necessary modules for unit testing, system operations, asynchronous programming, and mocking

current_script_path = os.path.abspath(__file__)
current_directory = os.path.dirname(current_script_path)
sys.path.append(current_directory)
sys.path.append(current_directory + "/../")
sys.path.append(current_directory + "/../../")
# 创建测试加载器
loader = unittest.TestLoader()
suite = unittest.TestSuite()

import logging

# 配置日志格式
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - %(message)s'
)

# 加载一个目录下的所有测试模块
suite.addTests(loader.discover(start_dir=current_directory, pattern='test_short_term_key.py'))

# 运行测试
runner = unittest.TextTestRunner()
runner.run(suite)



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import json
import os
from datetime import datetime, timezone
from typing import Any, Dict, List
import hashlib
import hmac
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives.asymmetric.utils import encode_dss_signature, decode_dss_signature
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives import constant_time
from didall.utils.crypto_tool import encrypt_aes_gcm_sha256, generate_16_char_from_random_num, generate_random_hex, generate_signature_for_json

def generate_register_message(version: str, 
                              routers: List[Dict[str, Any]]) -> Dict[str, Any]:
    """
    Generate a message for registering routers.
    :param routers: A list containing router information, each router should include router DID, nonce and proof.
    :return: A dictionary of the constructed registration message.
    """
    registration_message = {
        "version": version,
        "type": "register",
        "timestamp": datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
        "messageId": generate_random_hex(16),
        "routers": routers
    }
    return registration_message


# Generate SourceHello message
def generate_source_hello(version: str, session_id: str, source_private_key: ec.EllipticCurvePrivateKey, 
                          source_did: str, destination_did: str, random: str,
                          source_public_key_hex: str, key_share_list: List[Dict[str, Any]],
                          cipher_suite_list: List[str]) -> Dict[str, Any]:
        
    source_hello = {
        "version": version,
        "type": "sourceHello",
        "timestamp": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
        "messageId": generate_random_hex(16),
        "sessionId": session_id,
        "sourceDid": source_did,
        "destinationDid": destination_did,
        "verificationMethod": {
            "id": f"{source_did}#keys-1",
            "type": "EcdsaSecp256r1VerificationKey2019",
            "publicKeyHex": source_public_key_hex
        },
        "random": random,
        "supportedVersions": ["1.0"],
        "cipherSuites": cipher_suite_list,
        "supportedGroups": [
            "secp256r1"
            # "secp384r1",
            # "secp521r1"
        ],
        "keyShares": key_share_list
    }

    proof = {
        "type": "EcdsaSecp256k1Signature2019",
        "verificationMethod": f"{source_did}#keys-1",
        "created": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%SZ'),
    }
    
    source_hello["proof"] = proof
    
    # Sign the message using the source private key
    proof["proofValue"] = generate_signature_for_json(source_private_key, source_hello)

    source_hello["proof"] = proof

    return source_hello

# Generate DestinationHello message
def generate_destination_hello(version: str, session_id: str, source_private_key: ec.EllipticCurvePrivateKey, 
                               source_did: str, destination_did: str, random: str,
                               source_public_key_hex: str, key_share: Dict[str, Any],
                               cipher_suite: str) -> Dict[str, Any]:
    
    destination_hello = {
        "version": version,
        "type": "destinationHello",
        "timestamp": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
        "messageId": generate_random_hex(16),
        "sessionId": session_id,
        "sourceDid": source_did,
        "destinationDid": destination_did,
        "verificationMethod": {
            "id": f"{source_did}#keys-1",
            "type": "EcdsaSecp256r1VerificationKey2019",
            "publicKeyHex": source_public_key_hex
        },
        "random": random,
        "selectedVersion": version,
        "cipherSuite": cipher_suite,
        "keyShare": key_share
    }

    # Prepare signature
    proof = {
        "type": "EcdsaSecp256k1Signature2019",
        "verificationMethod": f"{source_did}#keys-1",
        "created": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%SZ'),
    }
    
    destination_hello["proof"] = proof
    
    # Sign the message using the source private key
    proof["proofValue"] = generate_signature_for_json(source_private_key, destination_hello)

    destination_hello["proof"] = proof

    return destination_hello


def generate_finished_message(version: str, session_id: str, source_did: str, destination_did: str, 
                              source_hello_random: str, destination_hello_random: str, 
                              data_secret: bytes) -> Dict[str, Any]:
    """
    Generate Finished message
    :param version: Current protocol version
    :param session_id: Session ID
    :param source_did: Source DID identifier
    :param destination_did: Destination DID identifier
    :param source_hello_random: Random number from source Hello message
    :param destination_hello_random: Random number from destination Hello message
    :param data_secret: Negotiated handshake key
    :return: Dictionary of the Finished message
    """
    # Generate secret_key_id
    secret_key_id = generate_16_char_from_random_num(source_hello_random, destination_hello_random)
    
    secret_key_id_dict = {
        "secretKeyId": secret_key_id,
    }
    # Generate verifyData
    verify_data_dict = encrypt_aes_gcm_sha256(json.dumps(secret_key_id_dict).encode(), data_secret)
    
    # Construct Finished message
    finished_message = {
        "version": version,
        "type": "finished",
        "timestamp": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
        "messageId": generate_random_hex(16),        
        "sessionId": session_id,
        "sourceDid": source_did,
        "destinationDid": destination_did,
        "verifyData": verify_data_dict
    }
    
    return finished_message

def generate_response_message(version: str, original_type: str, message_id: str, 
                              code: int, detail: str) -> Dict[str, Any]:
    """
    Generate response message
    :param version: Current protocol version
    :param original_type: Original message type
    :param code: Response code
    :param detail: Response details
    :return: Dictionary of the response message
    """
    response_message = {
        "version": version,
        "type": "response",
        "timestamp": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
        "messageId": message_id,
        "originalType": original_type,
        "code": code,
        "detail": detail
    }
    return response_message

def generate_encrypted_message(version: str,  message_id: str, source_did: str, 
                               destination_did: str, secret_key_id: str, 
                               data: bytes, data_secret: bytes) -> Dict[str, Any]:
    """
    Generate encrypted message
    """
    encrypted_data = encrypt_aes_gcm_sha256(data, data_secret)
    encrypted_message = {
        "version": version,
        "type": "message",
        "timestamp": datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z',
        "messageId": message_id,
        "sourceDid": source_did,
        "destinationDid": destination_did,
        "secretKeyId": secret_key_id,
        "encryptedData": encrypted_data
    }
    return encrypted_message






# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import logging
from typing import Tuple
import aiohttp
from didall.utils.crypto_tool import get_pem_from_private_key
from didall.utils.did_generate import did_generate
import requests

class DIDAllClient:
    def __init__(self, did_service_url: str, api_key: str):
        self.api_key = api_key
        self.did_service_url = did_service_url

    def generate_did_document(self, communication_service_endpoint: str, router_did: str = "") -> Tuple[str, str, str]:
        """
        Generate DID document without registering to DID service

        Args:
            communication_service_endpoint (str): Communication service endpoint for DID document.
            router (str, optional): Router's DID, default is empty string.

        Returns:
            Tuple[str, str, str]: A tuple containing private key in PEM format, DID string, and DID document JSON string.
        """
        private_key, _, did, did_document_json = did_generate(communication_service_endpoint, router_did)

        # Convert private key to PEM format
        private_key_pem = get_pem_from_private_key(private_key)

        return private_key_pem, did, did_document_json

    async def generate_register_did_document(self, communication_service_endpoint: str, router_did: str = "") -> Tuple[str, str, str]:
        """
        Register DID document to DID service.

        This function asynchronously generates DID and corresponding DID document, and registers it to the configured DID service.
        It uses the aiohttp library to send asynchronous HTTP requests.

        Args:
            communication_service_endpoint (str): Communication service endpoint for DID document.
            router_did (str, optional): Router's DID, default is empty string.

        Returns:
            Tuple[str, str, str]: A tuple containing private key in PEM format, DID string, and DID document JSON string.
            If registration fails, it will return three None values.
        """

        # Generate private key, public key, DID and DID document
        private_key, _, did, did_document_json = did_generate(communication_service_endpoint, router_did)

        # Convert private key to PEM format
        private_key_pem = get_pem_from_private_key(private_key)

        # Prepare request headers
        headers = {
            "Content-Type": "application/text",
            "Authorization": f"Bearer {self.api_key}"
        }

        # Ensure correct request URL
        request_url = f"{self.did_service_url}/v1/did"  # Make sure the URL is correct

        # Use aiohttp to send asynchronous POST request
        async with aiohttp.ClientSession() as session:
            async with session.post(request_url, headers=headers, data=did_document_json) as response:
                if response.status == 200:
                    return private_key_pem, did, did_document_json
                else:
                    response_text = await response.text()
                    logging.error(f"Failed to create DID document: {response.status} {response_text}")
                    return None, None, None

    async def get_did_document(self, did: str):
        # Prepare request headers
        headers = {
            "Accept": "application/text",
            "Authorization": f"Bearer {self.api_key}"
        }

        # Construct complete request URL
        request_url = f"{self.did_service_url}/v1/did/{did}"

        # Use aiohttp to send asynchronous GET request
        async with aiohttp.ClientSession() as session:
            async with session.get(request_url, headers=headers) as response:
                if response.status == 200:
                    return await response.text()
                else:
                    response_text = await response.text()
                    logging.error(f"Failed to retrieve DID document: {response.status} {response_text}")
                    return None

    def register_did_document_sync(self, communication_service_endpoint: str, router: str = ""):
        # Generate private key, public key, DID and DID document
        private_key, _, did, did_document_json = did_generate(communication_service_endpoint, router)

        # Convert private key to PEM format
        private_key_pem = get_pem_from_private_key(private_key)

        # Prepare request headers
        headers = {
            "Content-Type": "application/text",
            "Authorization": f"Bearer {self.api_key}"
        }

        # Use requests to send synchronous POST request
        response = requests.post(f"{self.did_service_url}/v1/did", headers=headers, data=did_document_json)
        if response.status_code == 200:
            return private_key_pem, did, did_document_json
        else:
            logging.error(f"Failed to create DID document: {response.status_code} {response.text}")
            return None, None, None

    def get_did_document_sync(self, did: str):
        # Prepare request headers
        headers = {
            "Accept": "application/text",
            "Authorization": f"Bearer {self.api_key}"
        }

        # Send synchronous GET request
        response = requests.get(f"{self.did_service_url}/v1/did/{did}", headers=headers)
        if response.status_code == 200:
            return response.text
        else:
            logging.error(f"Failed to retrieve DID document: {response.status_code} {response.text}")
            return None


# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


from datetime import datetime
import hashlib
import base64
import json
import os
import secrets
import logging
from typing import Any, Dict, Tuple
import base58
import struct
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric.utils import decode_dss_signature, encode_dss_signature
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.kdf.hkdf import HKDF, HKDFExpand
from cryptography.exceptions import InvalidSignature
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes

# Generate 32 bytes of random data
def generate_random_hex(length=32):
    return secrets.token_hex(length)

def generate_16_char_from_random_num(random_num1: str, random_num2: str):
    content = random_num1 + random_num2
    random_bytes = content.encode('utf-8')
    
    # Use HKDF to derive 8 bytes of random data
    hkdf = HKDF(
        algorithm=hashes.SHA256(),  # Ensure using hash algorithm instance from cryptography library
        length=8,  # Generate 8-byte key
        salt=None,
        info=b'',  # Optional context information to differentiate keys for different purposes
        backend=default_backend()  # Use default crypto backend
    )
    
    derived_key = hkdf.derive(random_bytes)
    
    # Encode the derived key as a hexadecimal string
    derived_key_hex = derived_key.hex()
    
    return derived_key_hex

def get_hex_from_public_key(public_key: ec.EllipticCurvePublicKey) -> str:
    """Convert EC public key to hexadecimal string"""
    # Get byte representation of public key
    public_key_bytes = public_key.public_bytes(
        encoding=serialization.Encoding.X962,
        format=serialization.PublicFormat.UncompressedPoint
    )
    # Convert bytes to hexadecimal string
    return public_key_bytes.hex()

def get_public_key_from_hex(public_key_hex: str, curve: ec.EllipticCurve = ec.SECP256R1()) -> ec.EllipticCurvePublicKey:
    """Extract EC public key from hexadecimal string, specifying elliptic curve"""
    # Convert hexadecimal string to bytes
    public_key_bytes = bytes.fromhex(public_key_hex)
    # Create public key from bytes
    return ec.EllipticCurvePublicKey.from_encoded_point(curve, public_key_bytes)


# Generate ECDSA private and public key pair
def generate_ec_key_pair(curve: ec.EllipticCurve=ec.SECP256R1()) -> Tuple[ec.EllipticCurvePrivateKey, ec.EllipticCurvePublicKey, str]:
    private_key = ec.generate_private_key(curve, default_backend())
    public_key = private_key.public_key()
    public_numbers = public_key.public_numbers()
    public_key_hex = '04' + format(public_numbers.x, '064x') + format(public_numbers.y, '064x')
    return private_key, public_key, public_key_hex

def generate_secp256r1_private_key() -> ec.EllipticCurvePrivateKey:
    """Generate Secp256r1 private key"""
    return ec.generate_private_key(ec.SECP256R1())

def generate_secp256r1_public_key(private_key: ec.EllipticCurvePrivateKey) -> ec.EllipticCurvePublicKey:
    """Generate public key from Secp256r1 private key"""
    return private_key.public_key()


def generate_bitcoin_address(public_key: ec.EllipticCurvePublicKey) -> str:
    """Generate Bitcoin address from public key"""
    public_key_bytes = public_key.public_bytes(
        encoding=serialization.Encoding.X962,
        format=serialization.PublicFormat.UncompressedPoint
    )
    sha256_pk = hashlib.sha256(public_key_bytes).digest()
    ripemd160_pk = hashlib.new('ripemd160', sha256_pk).digest()
    pubkey_hash = b'\x00' + ripemd160_pk
    checksum = hashlib.sha256(hashlib.sha256(pubkey_hash).digest()).digest()[:4]
    address = base58.b58encode(pubkey_hash + checksum).decode('utf-8')
    return address

def generate_signature_for_json(private_key: ec.EllipticCurvePrivateKey, did_document: Dict[str, Any]) -> str:
    # Convert data to be signed to JSON string
    did_document_str = json.dumps(did_document, separators=(',', ':'), sort_keys=True)
    did_document_bytes = did_document_str.encode('utf-8')
    
    # Sign data using private key
    signature = private_key.sign(did_document_bytes, ec.ECDSA(hashes.SHA256()))
    
    # Separate r and s values
    r, s = decode_dss_signature(signature)
    
    # Convert r and s to bytes
    r_bytes = r.to_bytes((r.bit_length() + 7) // 8, byteorder='big')
    s_bytes = s.to_bytes((s.bit_length() + 7) // 8, byteorder='big')
    
    # Base64 URL-safe encode
    proof_value = base64.urlsafe_b64encode(r_bytes + s_bytes).rstrip(b'=').decode('utf-8')
    
    return proof_value

def verify_signature_for_json(public_key: ec.EllipticCurvePublicKey, did_document: Dict[str, Any], signature: str) -> bool:
    """Verify signature of JSON message"""
    try:
        # Decode signature from Base64Url encoding to byte string
        signature_bytes = base64.urlsafe_b64decode(signature + '==')
        
        # Split r and s values
        r_length = len(signature_bytes) // 2
        r_bytes = signature_bytes[:r_length]
        s_bytes = signature_bytes[r_length:]
        
        r = int.from_bytes(r_bytes, byteorder='big')
        s = int.from_bytes(s_bytes, byteorder='big')
        
        # Reassemble into ASN.1 DER encoded format
        der_encoded_signature = encode_dss_signature(r, s)
        
        # Convert message to byte string
        message_str = json.dumps(did_document, separators=(',', ':'), sort_keys=True)
        message_bytes = message_str.encode('utf-8')
        
        # Verify signature
        public_key.verify(der_encoded_signature, message_bytes, ec.ECDSA(hashes.SHA256()))
        return True
    except Exception as e:
        logging.error(f"Signature verification failed: {e}")
        return False
    
def generate_router_json(private_key: ec.EllipticCurvePrivateKey, did_document: Dict[str, Any]) -> Dict[str, Any]:
    """Generate router JSON dictionary based on private key and DID document"""
    router_did = did_document.get("id")
    nonce = generate_random_hex(32)  # Assume there's a function generate_random_hex to generate random hex string
    current_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')

    # Get verificationMethod from DID document
    verification_methods = did_document.get("verificationMethod", [])
    verification_method_id = next((vm["id"] for vm in verification_methods if vm["type"] == "EcdsaSecp256r1VerificationKey2019"), None)

    # Create router dictionary
    router = {
        "router": router_did,
        "nonce": nonce,
        "proof": {
            "type": "EcdsaSecp256r1Signature2019",
            "created": current_time,
            "proofPurpose": "assertionMethod",
            "verificationMethod": verification_method_id
        }
    }

    # Sign router information using private key
    proof_value = generate_signature_for_json(private_key, router)
    router["proof"]["proofValue"] = proof_value

    return router

def verify_did_with_public_key(did: str, public_key: ec.EllipticCurvePublicKey) -> bool:
    """Verify public key based on DID"""
    try:
        # Extract Bitcoin address
        bitcoin_address = did.split(':')[-1]

        # Generate Bitcoin address from public key
        generated_address = generate_bitcoin_address(public_key)

        # Verify if generated Bitcoin address matches the address in DID
        return bitcoin_address == generated_address
    except Exception as e:
        return False

def get_pem_from_private_key(private_key: ec.EllipticCurvePrivateKey) -> str:
    """Get PEM format string from EllipticCurvePrivateKey"""
    pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption()
    )
    return pem.decode('utf-8')


# Load private key from PEM format string
def load_private_key_from_pem(pem_str: str) -> ec.EllipticCurvePrivateKey:
    return serialization.load_pem_private_key(
        pem_str.encode('utf-8'),
        password=None,
        backend=default_backend()
    )


def generate_shared_secret(private_key: ec.EllipticCurvePrivateKey, 
                           peer_public_key: ec.EllipticCurvePublicKey) -> bytes:
    # Generate shared secret using peer's public key and own private key
    shared_secret = private_key.exchange(ec.ECDH(), peer_public_key)
    return shared_secret

def hkdf_label(length: int, label: bytes, context: bytes) -> bytes:
    full_label = b"als10 " + label
    hkdf_label = struct.pack("!H", length) + struct.pack("!B", len(full_label)) + full_label + struct.pack("!B", len(context)) + context
    return hkdf_label

def get_key_length_from_cipher_suite(cipher_suite: str) -> int:
    if cipher_suite == "TLS_AES_128_GCM_SHA256":
        return 16
    elif cipher_suite == "TLS_AES_256_GCM_SHA384":
        return 32
    elif cipher_suite == "TLS_CHACHA20_POLY1305_SHA256":
        return 32
    else:
        raise ValueError(f"Unsupported cipher suite: {cipher_suite}")


def derive_tls13_data_keys(shared_secret: bytes, source_hello_random: bytes, 
                           destination_hello_random: bytes, 
                           key_length=16, hash_algorithm=hashes.SHA256()):
    backend = default_backend()

    # HKDF Extract stage
    hkdf_extract = HKDF(
        algorithm=hash_algorithm,
        length=hash_algorithm.digest_size,
        salt=b'\x00' * hash_algorithm.digest_size,  # Initial salt is zeros
        info=None,
        backend=backend
    )
    extracted_key = hkdf_extract.derive(shared_secret)

    # Derive-Secret function in TLS 1.3
    def derive_secret(secret: bytes, label: bytes, messages: bytes) -> bytes:
        hkdf_expand = HKDFExpand(
            algorithm=hash_algorithm,
            length=hash_algorithm.digest_size,
            info=hkdf_label(hash_algorithm.digest_size, label, messages),
            backend=backend
        )
        return hkdf_expand.derive(secret)

    # Generate handshake traffic secrets
    source_data_traffic_secret = derive_secret(extracted_key, b"s ap traffic", source_hello_random + destination_hello_random)
    destination_data_traffic_secret = derive_secret(extracted_key, b"d ap traffic", source_hello_random + destination_hello_random)

    # Expand to generate actual handshake keys
    source_data_key = HKDF(
        algorithm=hash_algorithm,
        length=key_length,  # 256-bit key for AES-256
        salt=None,
        info=hkdf_label(32, b"key", source_data_traffic_secret),
        backend=backend
    ).derive(source_data_traffic_secret)

    destination_data_key = HKDF(
        algorithm=hash_algorithm,
        length=key_length,  # 256-bit key for AES-256
        salt=None,
        info=hkdf_label(32, b"key", destination_data_traffic_secret),
        backend=backend
    ).derive(destination_data_traffic_secret)

    # Return derived handshake keys and secrets
    return (source_data_key, destination_data_key, source_data_traffic_secret, destination_data_traffic_secret)

# Generate TLS1.3 application layer keys, not used for now
def derive_tls13_application_keys(client_handshake_traffic_secret, server_handshake_traffic_secret, hash_algorithm=hashes.SHA256()):
    backend = default_backend()

    # Derive application traffic secrets
    client_application_traffic_secret = HKDFExpand(
        algorithm=hash_algorithm,
        length=hash_algorithm.digest_size,
        info=hkdf_label(hash_algorithm.digest_size, b"c ap traffic", b""),
        backend=backend
    ).derive(client_handshake_traffic_secret)

    server_application_traffic_secret = HKDFExpand(
        algorithm=hash_algorithm,
        length=hash_algorithm.digest_size,
        info=hkdf_label(hash_algorithm.digest_size, b"s ap traffic", b""),
        backend=backend
    ).derive(server_handshake_traffic_secret)

    # Expand to generate actual application keys
    client_application_key = HKDF(
        algorithm=hash_algorithm,
        length=32,  # 256-bit key for AES-256
        salt=None,
        info=hkdf_label(32, b"key", client_application_traffic_secret),
        backend=backend
    ).derive(client_application_traffic_secret)

    server_application_key = HKDF(
        algorithm=hash_algorithm,
        length=32,  # 256-bit key for AES-256
        salt=None,
        info=hkdf_label(32, b"key", server_application_traffic_secret),
        backend=backend
    ).derive(server_application_traffic_secret)

    # Return derived application keys and secrets
    return {
        'client_application_key': client_application_key,
        'server_application_key': server_application_key,
        'client_application_traffic_secret': client_application_traffic_secret,
        'server_application_traffic_secret': server_application_traffic_secret
    }

# TLS_AES_128_GCM_SHA256 encryption function
def encrypt_aes_gcm_sha256(data: bytes, key: bytes) -> Dict[str, str]:
    # Ensure key length is 16 bytes (128 bits)
    if len(key) != 16:
        raise ValueError("Key must be 128 bits (16 bytes).")
    
    # 生成随机IV
    iv = os.urandom(12)  # 对于GCM，推荐的IV长度是12字节
    
    # 创建加密对象
    encryptor = Cipher(
        algorithms.AES(key),
        modes.GCM(iv),
        backend=default_backend()
    ).encryptor()
    
    # 加密数据
    ciphertext = encryptor.update(data) + encryptor.finalize()
    
    # 获取tag
    tag = encryptor.tag
    
    # 编码为Base64
    iv_encoded = base64.b64encode(iv).decode('utf-8')
    tag_encoded = base64.b64encode(tag).decode('utf-8')
    ciphertext_encoded = base64.b64encode(ciphertext).decode('utf-8')
    
    # 创建JSON对象
    encrypted_data = {
        "iv": iv_encoded,
        "tag": tag_encoded,
        "ciphertext": ciphertext_encoded
    }
        
    return encrypted_data

def decrypt_aes_gcm_sha256(encrypted_json: Dict[str, str], key: bytes) -> str:
    # Base64解码
    iv = base64.b64decode(encrypted_json["iv"])
    ciphertext = base64.b64decode(encrypted_json["ciphertext"])
    tag = base64.b64decode(encrypted_json["tag"])
    
    # 创建解密对象
    decryptor = Cipher(
        algorithms.AES(key),
        modes.GCM(iv, tag),
        backend=default_backend()
    ).decryptor()
    
    # 解密数据
    plaintext = decryptor.update(ciphertext) + decryptor.finalize()
    
    return plaintext.decode()





# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.

import os
import hashlib
import base58
import base64
import json
import datetime
from typing import Dict, Any, Tuple
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric.utils import encode_dss_signature
from didall.utils.crypto_tool import generate_secp256r1_private_key, \
                              generate_secp256r1_public_key, \
                              generate_bitcoin_address, generate_signature_for_json

def generate_did(bitcoin_address: str) -> str:
    """Generate DID based on Bitcoin address"""
    return f"did:all:{bitcoin_address}"

def create_did_document(did: str, public_key: ec.EllipticCurvePublicKey, 
                        service_endpoint: str, router: str) -> Dict[str, Any]:
    """Generate DID document"""
    public_key_bytes = public_key.public_bytes(
        encoding=serialization.Encoding.X962,
        format=serialization.PublicFormat.UncompressedPoint
    )
    public_key_hex = '04' + public_key_bytes.hex()
    return {
        "@context": "https://www.w3.org/ns/did/v1",
        "id": did,
        "controller": did,
        "verificationMethod": [
            {
                "id": f"{did}#keys-1",
                "type": "EcdsaSecp256r1VerificationKey2019",
                "controller": did,
                "publicKeyHex": public_key_hex
            }
        ],
        "authentication": [
            {
                "id": f"{did}#keys-1",
                "type": "EcdsaSecp256r1VerificationKey2019",
                "controller": did,
                "publicKeyHex": public_key_hex
            }
        ],
        "service": [
            {
                "id": f"{did}#communication",
                "type": "messageService",
                "router": router,
                "serviceEndpoint": service_endpoint
            }
        ]
    }

def sign_did_document_secp256r1(private_key: ec.EllipticCurvePrivateKey, did_document: Dict[str, Any]) -> Dict[str, Any]:
    """Sign the DID document"""

    # Add timestamp
    created = datetime.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
    proof = {
        "type": "EcdsaSecp256r1Signature2019",
        "created": created,
        "proofPurpose": "assertionMethod",
        "verificationMethod": f"{did_document['id']}#keys-1",
    }

    # Add proof to DID document for signing
    did_document['proof'] = proof

    proof_value = generate_signature_for_json(private_key, did_document)

    # Add signature to proof
    proof['proofValue'] = proof_value
    did_document['proof'] = proof

    return did_document

def did_generate(communication_service_endpoint: str, router: str="") -> \
                Tuple[ec.EllipticCurvePrivateKey, ec.EllipticCurvePublicKey, str, str]:
    private_key = generate_secp256r1_private_key()
    public_key = generate_secp256r1_public_key(private_key)
    bitcoin_address = generate_bitcoin_address(public_key)

    did = generate_did(bitcoin_address)
    if not router:
        router = did

    did_document = create_did_document(did, public_key, communication_service_endpoint, router)
    signed_did_document = sign_did_document_secp256r1(private_key, did_document)
    did_document_json = json.dumps(signed_did_document, indent=2)
    return private_key, public_key, did, did_document_json



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


import unittest
import asyncio
import json
from typing import Dict, Any
from cryptography.hazmat.primitives import serialization
import sys
import os
import logging

current_script_path = os.path.abspath(__file__)
current_directory = os.path.dirname(current_script_path)
sys.path.append(current_directory)
sys.path.append(current_directory + "/../")
sys.path.append(current_directory + "/../../")

from didall.short_term_key_generater import ShortTermKeyGenerater
from didall.utils.did_generate import did_generate

class TestShortTermKeyGeneration(unittest.TestCase):
    def setUp(self):
        # Generate DID and private key for initiator and receiver using did_generate
        initiator_private_key, initiator_public_key, initiator_did, _ = did_generate("wss://initiator.com/endpoint")
        receiver_private_key, receiver_public_key, receiver_did, _ = did_generate("wss://receiver.com/endpoint")

        # Save private keys in PEM format
        initiator_private_key_pem = initiator_private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        receiver_private_key_pem = receiver_private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )

        initiator_private_key_pem = initiator_private_key_pem.decode('utf-8')
        receiver_private_key_pem = receiver_private_key_pem.decode('utf-8')

        # Initialize two instances, one as initiator and one as receiver
        self.initiator = ShortTermKeyGenerater(
            local_did=initiator_did,
            did_private_key_pem=initiator_private_key_pem,
            remote_did=receiver_did,
            json_send_func=self.receiver_receive_json,
            is_initiator=True
        )

        self.receiver = ShortTermKeyGenerater(
            local_did=receiver_did,
            did_private_key_pem=receiver_private_key_pem,
            remote_did=initiator_did,
            json_send_func=self.initiator_receive_json,
            is_initiator=False,
            session_id=self.initiator.session_id
        )

        self.initiator_did = initiator_did
        self.receiver_did = receiver_did

    async def initiator_receive_json(self, json_data: Dict[str, Any]):
        # Wait for 10ms
        self.initiator.receive_json_message(json_data)
        await asyncio.sleep(0.01)

    async def receiver_receive_json(self, json_data: Dict[str, Any]):
        self.receiver.receive_json_message(json_data)
        await asyncio.sleep(0.01)

    async def initiate_key_exchange(self):
        result = await self.initiator.generate_short_term_key_active()
        logging.info(f'generate_short_term_key_active {result}')
        self.assertTrue(result, "Key exchange failed")
    
    async def receive_key_exchange(self):
        """Start the key exchange process"""
        result = await self.receiver.generate_short_term_key_passive()
        logging.info(f'generate_short_term_key_passive {result}')
        self.assertTrue(result, "Key exchange failed")


    def test_key_exchange(self):
        # Test key exchange process
        async def run_test():
            return await asyncio.gather(
                self.initiate_key_exchange(),
                self.receive_key_exchange()
            )

        asyncio.run(run_test())

        # Verify if both parties have negotiated the same key
        initiator_remote_did, \
        initiator_send_encryption_key, \
        initiator_receive_decryption_key, \
        initiator_secret_key_id, \
        initiator_key_expires, \
        initiator_cipher_suite = self.initiator.get_final_short_term_key()

        receiver_remote_did, \
        receiver_send_encryption_key, \
        receiver_receive_decryption_key, \
        receiver_secret_key_id, \
        receiver_key_expires, \
        receiver_cipher_suite = self.receiver.get_final_short_term_key()

        self.assertEqual(initiator_remote_did, self.receiver_did, "Remote DID mismatch")
        self.assertEqual(receiver_remote_did, self.initiator_did, "Remote DID mismatch")
        self.assertEqual(initiator_send_encryption_key, receiver_receive_decryption_key, "Send encryption key mismatch")
        self.assertEqual(initiator_receive_decryption_key, receiver_send_encryption_key, "Receive decryption key mismatch")
        self.assertEqual(initiator_secret_key_id, receiver_secret_key_id, "Secret key ID mismatch")
        self.assertEqual(initiator_key_expires, receiver_key_expires, "Key expiration mismatch")
        self.assertEqual(initiator_cipher_suite, receiver_cipher_suite, "Cipher suite mismatch")

        # logging.info(f"initiator_remote_did: {initiator_remote_did}")
        # logging.info(f"receiver_remote_did: {receiver_remote_did}")
        # logging.info(f"initiator_send_encryption_key: {initiator_send_encryption_key}")
        # logging.info(f"initiator_receive_decryption_key: {initiator_receive_decryption_key}")
        # logging.info(f"initiator_secret_key_id: {initiator_secret_key_id}")
        # logging.info(f"receiver_send_encryption_key: {receiver_send_encryption_key}")
        # logging.info(f"receiver_receive_decryption_key: {receiver_receive_decryption_key}")
        # logging.info(f"receiver_secret_key_id: {receiver_secret_key_id}")
        # logging.info(f"initiator_key_expires: {initiator_key_expires}")
        # logging.info(f"receiver_key_expires: {receiver_key_expires}")
        # logging.info(f"initiator_cipher_suite: {initiator_cipher_suite}")
        # logging.info(f"receiver_cipher_suite: {receiver_cipher_suite}")

if __name__ == "__main__":
    unittest.main()




# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.

from setuptools import setup, find_packages

setup(
    name="didall",
    version="0.1.4",
    packages=find_packages(),
    install_requires=[
        'ecdsa>=0.19.0', 
        'cryptography>=42.0.8', 
        'asn1crypto>=1.5.1', 
        'base58>=2.1.1', 
        'aiohttp>=3.9.5', 
        'requests>=2.32.3', 
        'websockets>=12.0'
    ],
    author="GaoWei Chang",
    author_email="chgaowei@gmail.com",
    description="An SDK for cross-platform identity authentication and end-to-end encrypted communication based on DID",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://gitee.com/bitaimeta_admin/didall",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.


from datetime import datetime, timezone
import asyncio
import traceback
import os
import sys
import logging
from copy import deepcopy
from typing import Callable, List, Dict, Any
import json
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from cryptography.exceptions import InvalidSignature


current_script_path = os.path.abspath(__file__)
current_directory = os.path.dirname(current_script_path)
sys.path.append(current_directory)
sys.path.append(current_directory + "/../")
sys.path.append(current_directory + "/../../")

from utils.crypto_tool import decrypt_aes_gcm_sha256, derive_tls13_data_keys, generate_16_char_from_random_num, generate_random_hex, generate_ec_key_pair, generate_shared_secret, get_key_length_from_cipher_suite, get_public_key_from_hex, load_private_key_from_pem, verify_did_with_public_key, verify_signature_for_json
from message_generation import generate_destination_hello, generate_finished_message, generate_source_hello

class ECKeyPair:
    def __init__(self, curve: ec.EllipticCurve):
        self.curve = curve
        self.private_key: ec.EllipticCurvePrivateKey = None
        self.public_key: ec.EllipticCurvePublicKey = None
        self.public_key_hex: str = None

        self.private_key, self.public_key, self.public_key_hex = generate_ec_key_pair(curve)

class ShortTermKeyGenerater:
    def __init__(self, local_did: str, did_private_key_pem: str, 
                 remote_did: str, json_send_func: Callable[[Dict[str, Any]], None],
                 is_initiator: bool, session_id: str=None):
        self.session_id: str = session_id if session_id else generate_random_hex(16)
        self.json_send_func: Callable[[Dict[str, Any]], None] = json_send_func
        self.is_initiator: bool = is_initiator
        self.event: asyncio.Event = asyncio.Event()

        self.source_hello_message: Dict[str, Any] = None
        self.destination_hello_message: Dict[str, Any] = None
        self.finished_message: Dict[str, Any] = None

        # Local information
        self.local_did: str = local_did
        self.local_did_private_key: ec.EllipticCurvePrivateKey = None
        self.local_did_public_key_hex: str = None

        self.local_ec_key_pair: ECKeyPair = ECKeyPair(ec.SECP256R1())
        self.local_short_key_expires: int = 864000
        self.local_key_share: Dict[str, Any] = {
            "group": self.local_ec_key_pair.curve.name,
            "expires": self.local_short_key_expires,  # Example: 10 days
            "keyExchange": self.local_ec_key_pair.public_key_hex
        }
        self.local_cipher_suites: List[str] = ["TLS_AES_128_GCM_SHA256"]  #  "TLS_AES_128_GCM_SHA256" "TLS_AES_256_GCM_SHA384" "TLS_CHACHA20_POLY1305_SHA256"
        self.set_local_key_from_pem(did_private_key_pem)

        # Remote information
        self.remote_did: str = remote_did
        self.remote_did_public_key: ec.EllipticCurvePublicKey = None
        self.remote_key_share: Dict[str, Any] = {}

        # Current state
        # As initiator, states include: init, wait_destination_hello, wait_finished, finished
        # As responder, states include: init, wait_finished, finished
        self.state: str = "init"   

        # Encryption keys
        self.source_hello_random: str = generate_random_hex(32) if self.is_initiator else None
        self.destination_hello_random: str = None if self.is_initiator else generate_random_hex(32)
        self.send_encryption_key: bytes = None  # Encryption key for sending
        self.receive_decryption_key: bytes = None  # Decryption key for receiving
        self.secret_key_id: str = None
        self.key_expires: int = None
        self.cipher_suite: str = None

    def get_final_short_term_key(self):
        return  self.remote_did, \
                self.send_encryption_key, \
                self.receive_decryption_key, \
                self.secret_key_id, \
                self.key_expires, \
                self.cipher_suite

    def set_local_key_from_pem(self, pem_data):
        self.local_did_private_key = load_private_key_from_pem(pem_data)
        public_key = self.local_did_private_key.public_key()
        public_numbers = public_key.public_numbers()
        self.local_did_public_key_hex = '04' + format(public_numbers.x, '064x') + format(public_numbers.y, '064x')
    

    async def send_source_hello(self):
        # Generate SourceHello message
        source_hello = generate_source_hello(
            "1.0",
            self.session_id,
            self.local_did_private_key,
            self.local_did,
            self.remote_did,
            self.source_hello_random,
            self.local_did_public_key_hex,
            [self.local_key_share],
            self.local_cipher_suites
        )
        
        # Send message
        self.source_hello_message = source_hello
        await self.json_send_func(source_hello)

    async def send_destination_hello(self):
        destination_hello = generate_destination_hello(
            "1.0",
            self.session_id,
            self.local_did_private_key,
            self.local_did,
            self.remote_did,
            self.destination_hello_random,
            self.local_did_public_key_hex,
            self.local_key_share,
            self.cipher_suite
        )
        # Send message
        self.destination_hello_message = destination_hello
        await self.json_send_func(destination_hello)

    async def send_finished(self):
        finished_message = generate_finished_message(
            "1.0",
            self.session_id,
            self.local_did,
            self.remote_did,
            self.source_hello_random,
            self.destination_hello_random,
            self.send_encryption_key
        )
        await self.json_send_func(finished_message)

    def receive_json_message(self, message_json: Dict[str, Any]):
        if message_json.get('type') == "sourceHello":
            self.source_hello_message = message_json
        elif message_json.get('type') == "destinationHello":
            self.destination_hello_message = message_json
            self.event.set()
            self.event.clear()
        elif message_json.get('type') == "finished":
            self.finished_message = message_json
            logging.info(f"receive_json_message[{id(self)}], finished: {message_json}")
            self.event.set()
            self.event.clear()
        else:
            logging.error("Unknown message type")


    def check_message_valid(self, message: Dict[str, Any]):
        session_id = message['sessionId']
        source_did = message['sourceDid']
        destination_did = message['destinationDid']
        if source_did != self.remote_did:
            logging.error(f"sourceDid does not match local DID: {source_did}, remote_did: {self.remote_did}")
            return False
        if destination_did != self.local_did:
            logging.error(f"destinationDid does not match local DID: {destination_did}, local_did: {self.local_did}")
            return False
        if session_id != self.session_id:
            logging.error(f"sessionId does not match local sessionId: {session_id}, session_id: {self.session_id}")
            return False
        
        return True

    def check_message_did_public_key(self, source_hello: Dict[str, Any], source_public_key: ec.EllipticCurvePublicKey):
        # 2. Verify DID and public key correspondence
        source_did = source_hello['sourceDid']
        is_did_valid = verify_did_with_public_key(source_did, source_public_key)
        if not is_did_valid:
            logging.error("DID verification failed")
            return False
        return True

    def check_message_proof(self, source_hello: Dict[str, Any], source_public_key: ec.EllipticCurvePublicKey):
        # 3. Verify proof signature
        original_message = deepcopy(source_hello)
        del original_message['proof']['proofValue']

        proof = source_hello["proof"]
        proof_value = proof['proofValue']
        is_signature_valid = verify_signature_for_json(source_public_key, original_message, proof_value)
        if not is_signature_valid:
            logging.error("Signature verification failed")
            return False
        
        created_time_str = proof['created']
        created_time = datetime.strptime(created_time_str, "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
        current_time = datetime.now(timezone.utc)
        time_difference = (current_time - created_time).total_seconds()

        if abs(time_difference) > 30:
            logging.error("Replay attack detected, proof timestamp differs from current time by more than 30 seconds")
            return False

        return True

    def process_source_hello(self):
        if not self.source_hello_message:
            logging.error("source_hello_message is empty")
            return False
        
        try:
            source_hello = self.source_hello_message

            # 1. Verify DID and session id
            if not self.check_message_valid(source_hello):
                return False

            verification_method = source_hello['proof']['verificationMethod']
            source_public_key = self.extract_public_key(source_hello, verification_method)

            # 2. Verify DID and public key correspondence
            if not self.check_message_did_public_key(source_hello, source_public_key):
                logging.error("DID verification failed")
                return False

            # 3. Verify proof signature
            if not self.check_message_proof(source_hello, source_public_key):
                logging.error("Signature verification failed")
                return False

            # 4. Record remote information, including random number, key share, etc.
            self.source_hello_random = source_hello['random']
            self.remote_did_public_key = source_public_key

            cipher_suites = source_hello['cipherSuites']
            for cs in self.local_cipher_suites:
                if cs in cipher_suites:
                    self.cipher_suite = cs
                    break
            if not self.cipher_suite:
                logging.error(f"Unsupported cipher suite: {cipher_suites}, Local supported cipher suites: {self.local_cipher_suites}")
                return False

            for ks in source_hello['keyShares']:
                if ks['group'] == "secp256r1":
                    self.remote_key_share = ks
                    break
            if not self.remote_key_share:
                logging.error("secp256r1 not found")
                return False
            
            key_expires = int(self.remote_key_share['expires'])
            self.key_expires = min(key_expires, self.local_short_key_expires)

            logging.info("SourceHello message processed successfully")
            return True

        except Exception as e:
            logging.error(f"Error processing SourceHello message: {e}")
            logging.error("Exception stack trace:")
            traceback.print_exc()
            logging.error(f"Error processing SourceHello message: {e}")
            return False
        

    def process_destination_hello(self):
        if not self.destination_hello_message:
            logging.error("destination_hello_message is empty")
            return False
        
        try:
            destination_hello = self.destination_hello_message

            # 1. Verify DID and session id
            if not self.check_message_valid(destination_hello):
                return False
            
            verification_method = destination_hello['proof']['verificationMethod']
            source_public_key = self.extract_public_key(destination_hello, verification_method)

            # 2. Verify DID and public key correspondence
            if not self.check_message_did_public_key(destination_hello, source_public_key):
                logging.error("DID verification failed")
                return False

            # 3. Verify proof signature
            if not self.check_message_proof(destination_hello, source_public_key):
                logging.error("Signature verification failed")
                return False

            # 4. Record remote information, including random number, key share, etc.
            self.destination_hello_random = destination_hello['random']
            self.remote_did_public_key = source_public_key
            self.cipher_suite = destination_hello['cipherSuite']
            self.remote_key_share = destination_hello['keyShare']
            if self.remote_key_share['group'] != "secp256r1":
                logging.error("Unsupported elliptic curve group")
                return False
            
            key_expires = int(self.remote_key_share['expires'])
            self.key_expires = min(key_expires, self.local_short_key_expires)

            logging.info("DestinationHello message processed successfully")
            return True

        except Exception as e:
            logging.error(f"Error processing DestinationHello message: {e}")
            return False
    
    def generate_keys(self):
        remote_public_ec_key = get_public_key_from_hex(self.remote_key_share['keyExchange'])
        shared_secret = generate_shared_secret(self.local_ec_key_pair.private_key, 
                                               remote_public_ec_key)
        
        key_length = get_key_length_from_cipher_suite(self.cipher_suite)

        self.send_encryption_key, \
        self.receive_decryption_key, _, _ = derive_tls13_data_keys(shared_secret, 
                                                                    self.source_hello_random.encode('utf-8'), 
                                                                    self.destination_hello_random.encode('utf-8'),
                                                                    key_length=key_length)
        if not self.is_initiator:
            self.send_encryption_key, self.receive_decryption_key = (self.receive_decryption_key, self.send_encryption_key)


    def process_finished(self):
        if not self.finished_message:
            logging.error("finished_message is empty")
            return False

        if not self.check_message_valid(self.finished_message):
            return False

        verify_data = self.finished_message['verifyData']
        if not verify_data:
            logging.error("verifyData为空")
            return False

        content= decrypt_aes_gcm_sha256(verify_data, self.receive_decryption_key)
        content_dict = json.loads(content)
        if not content_dict:
            logging.error("ciphertext为空")
            return False
        
        secret_key_id = content_dict['secretKeyId']
        if not secret_key_id:
            logging.error("secretKeyId为空")
            return False
        
        expected_secret_key_id = generate_16_char_from_random_num(self.source_hello_random, self.destination_hello_random)

        if secret_key_id != expected_secret_key_id:
            logging.error("secretKeyId不匹配")
            return False
        
        self.secret_key_id = secret_key_id

        logging.info("Finished消息处理成功")
        return True

    def extract_public_key(self, did_document: Dict[str, Any], key_id: str) -> ec.EllipticCurvePublicKey:
        """从DID文档中提取公钥"""
        vm = did_document['verificationMethod']
        if vm['id'] == key_id and vm['type'] == "EcdsaSecp256r1VerificationKey2019":
            public_key_hex = vm['publicKeyHex']
            # 确保公钥以 '04' 开头
            if not public_key_hex.startswith('04'):
                logging.error(f"公钥必须以 '04' 开头: {public_key_hex}")
                raise ValueError("公钥必须以 '04' 开头")
            
            public_key_bytes = bytes.fromhex(public_key_hex)  # 确保从正确的位置开始转换
            return ec.EllipticCurvePublicKey.from_encoded_point(ec.SECP256R1(), public_key_bytes)
        
        raise ValueError(f"在DID文档中未找到公钥 {key_id}")

    # TODO: 捕获异常，发送错误消息
    async def generate_short_term_key_active(self):

        await self.send_source_hello()

        self.state = "wait_destination_hello"

        if not self.destination_hello_message:
            try:
                # 等待destination_hello消息到来
                await asyncio.wait_for(self.event.wait(), timeout=10)
            except asyncio.TimeoutError:
                if not self.destination_hello_message:
                    logging.error("wait destination hello timeout!")
                    return False
        
        # 处理destination_hello消息
        if not self.process_destination_hello():
            return False
        
        self.generate_keys()

        await self.send_finished()

        # 更新状态
        self.state = "wait_finished"

        if not self.finished_message:
            try:
                # 等待finished消息到来
                await asyncio.wait_for(self.event.wait(), timeout=10)
            except asyncio.TimeoutError:
                logging.error("wait finished timeout!")
                return False
        
        if not self.process_finished():
            return False

        self.state = "finished"
        logging.info(f"generate_short_term_key_active, success, secret_key_id: {self.secret_key_id}")
        return True
    
    async def generate_short_term_key_passive(self):
        # self.source_hello_message = source_hello_json

        self.state = "init"

        if not self.process_source_hello():
            return False
        
        await self.send_destination_hello()

        self.generate_keys()

        await self.send_finished()

        # 更新状态
        self.state = "wait_finished"

        if not self.finished_message:
            try:
                # 等待finished消息到来
                await asyncio.wait_for(self.event.wait(), timeout=10)
            except asyncio.TimeoutError:
                logging.error("generate_short_term_key_passive wait finished timeout!")
                return False
        
        if not self.process_finished():
            return False

        self.state = "finished"
        logging.info(f"generate_short_term_key_passive, success, secret_key_id: {self.secret_key_id}")
        return True
        

# 示例用法
if __name__ == "__main__":
    print('')



# Cross-Platform Identity Authentication and End-to-End Encrypted Communication Technology Based on DID
# Author: GaoWei Chang
# Email: chgaowei@gmail.com
# Website: http://pi-unlimited.com
#
# This project is open-sourced under the MIT License. For details, please see the LICENSE file.

import json
import hashlib
import base64
import base58
from typing import Dict, Any
from copy import deepcopy
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.exceptions import InvalidSignature
from didall.utils.did_generate import generate_bitcoin_address
from didall.utils.crypto_tool import verify_did_with_public_key, verify_signature_for_json

def extract_public_key(did_document: Dict[str, Any], key_id: str) -> ec.EllipticCurvePublicKey:
    """Extract public key from DID document"""
    for vm in did_document['verificationMethod']:
        if vm['id'] == key_id and vm['type'] == "EcdsaSecp256r1VerificationKey2019":
            public_key_hex = vm['publicKeyHex']
            public_key_bytes = bytes.fromhex(public_key_hex[2:])
            return ec.EllipticCurvePublicKey.from_encoded_point(ec.SECP256R1(), public_key_bytes)
    raise ValueError(f"Public key {key_id} not found in DID document")

def verify_did_document(did_document: dict) -> (bool, str):
    """Verify DID document"""
    try:
        # Extract signature-related data
        proof = did_document['proof']
        verification_method = proof['verificationMethod']
        signature = proof['proofValue']

        # Extract public key
        public_key = extract_public_key(did_document, verification_method)

        # Extract DID
        did = did_document['id']

        # Verify public key and DID
        is_did_valid = verify_did_with_public_key(did, public_key)
        if not is_did_valid:
            return False, "public key is not valid"

        # Verify signature
        # Remove proof field to get original message
        original_message = deepcopy(did_document)
        del original_message['proof']['proofValue']
        is_signature_valid = verify_signature_for_json(public_key, original_message, signature)

        return is_signature_valid, "verify signature error" if not is_signature_valid else ""
    except ValueError as ve:
        return False, str(ve)
    except InvalidSignature:
        return False, "Bad Signature Error"

